//Controller ////////////////////////////////////////////////////////////////////////////////////////////

function pageInit(){
	//connect code to the UI
	credentialForm = new CredentialForm(document.credentials);
	credentialsTable = new CredentialsTable();
	domainsTable = new DomainsTable();

	//Initialize the model and populate the ui
	deleteCredentials = new Array();

	var creds = retrieveCredentials();
	for(var i = 0; i < creds.length; ++i){
		credentialsTable.addRow(creds[i]);
	}
	retrieveDomains();
}

function doDelete(){
	//remove the table row that the delete button is in
	var row = this.parentNode.parentNode;
	row.parentNode.removeChild(row);
	if(row.id != 0){
		deleteCredentials.push(row.id);
	}else{
		//remove the auto-added domain row
		row.domainRow.parentNode.removeChild(row.domainRow);
	}
}

function addNewCredentials(){
	//insert newly added credentials into the database
	var credentialsRows = document.getElementById("credentialList").children;
	for(var i = 0; i < credentialsRows.length; ++i){
		if(credentialsRows[i].id == 0){
			//row is new
			addCredentials(new Credentials(credentialsRows[i].username,
				credentialsRows[i].password, credentialsRows[i].id));
		}
	}
}

function removeOldCredentials(){
	//remove deleted credentials from the database
	for(var i = 0; i < deleteCredentials.length; ++i){
		removeCredentials(deleteCredentials[i]);
	}
}

function changeQueryStates(){
	//update the database's domains table with all the checkboxes whose states have changed
	var tbody = document.getElementById("domainList");
	for(var i = 0; i < tbody.children.length; ++i){
		var checkbox = tbody.children[i].firstChild.firstChild;
		if(checkbox.checked != checkbox.initialState){
			setDomainEnabled(checkbox.domainId, checkbox.checked);
		}
	}
}


//CredentialForm class - deals with form validation, etc.  //////////////
function CredentialForm(form) {
	this.myForm = form;
	var o = this;
	this.myForm.add.onclick = function(){ o.doAdd(); };
}

CredentialForm.prototype.validate = function(){
	if(this.myForm.username.value.search(arguments.callee.validUsername) < 0){
		throw new Error(nrc_sz_ErrorMessage);
	}
	/*
	if(this.myForm.password.value.search(arguments.callee.validPassword) < 0){
		throw new Error("Invalid password");
	}
	*/
}
CredentialForm.prototype.validate.validUsername = /.+(\\|@).+/;
//CredentialForm.prototype.validate.validPassword = /.*/;

CredentialForm.prototype.reset = function(){
	this.myForm.reset();
}

CredentialForm.prototype.doAdd = function(){
	try{
		this.validate();
	}catch(e){
		alert(e.message);
		return;
	}

	var cred = new Credentials(this.myForm.username.value,
		this.myForm.password.value);
	credentialsTable.addRow(cred);
	credentialsTable.tbody.lastChild.domainRow = domainsTable.addRow(cred.getDomain());

	this.reset();
}


//View  /////////////////////////////////////////////////////////////////////////////////////////////////

function createData(content){
	var td = document.createElement("td");
	if(content != undefined) {
		var span = document.createElement("span");
		span.className = "dataDisplay";
		span.appendChild(document.createTextNode(content));
		td.appendChild(span);
	}
	return td;
}

//CredentialsTable class - manipulate the credentials table //////////
function CredentialsTable(){
	this.tbody = document.getElementById("credentialList");
}

CredentialsTable.prototype.addRow = function(cred){
	var row = this._createRow(cred);
	this.tbody.appendChild(row);

	var section = document.getElementById("credListDiv");
	section.style.visibility = "visible";
}

CredentialsTable.prototype._createRow = function(cred){
	var row = document.createElement("tr");
	row.appendChild(createData(cred.username));

	var deleteCell = createData();
	var button = document.createElement("input");
	button.setAttribute("type", "button");
	button.setAttribute("name", "delete" + cred.getDomain());
	button.setAttribute("value", nrc_sz_RemoveButton);
	button.domain = cred.getDomain();
	button.onclick = doDelete;
	deleteCell.appendChild(button);
	row.appendChild(deleteCell);

	row.username = cred.username;
	row.password = cred.password;
	row.id = cred.id;

	return row;
}

//DomainsTable class - manipulate the domain list table ////////////////

function DomainsTable(){
	this.tbody = document.getElementById("domainList");
}

DomainsTable.prototype.addRow = function (domain, enabled, domainId){
	var row = document.createElement("tr");

	var checkboxCell = createData();
	var checkbox = document.createElement("input");
	checkbox.setAttribute("type", "checkbox");
	checkbox.setAttribute("name", "query" + domain);
	checkbox.domain = domain;
	checkbox.domainId = domainId;
	checkboxCell.appendChild(checkbox);
	row.appendChild(checkboxCell);

	row.appendChild(createData(domain));

	this.tbody.appendChild(row);
	checkbox = this.tbody.lastChild.firstChild.firstChild;

	//set the enabled state, and remember the initial state for later
	checkbox.checked = (enabled == 1 || enabled == undefined) ? true : false;
	checkbox.initialState = checkbox.checked;

	var section = document.getElementById("domainListDiv");
	section.style.visibility = "visible";

	return row;
}


//Model  ///////////////////////////////////////////////////////////////////////////////////////////////////
function parseXmlString(xmlStr){
	var xmlDoc=new ActiveXObject("Microsoft.XMLDOM")
	xmlDoc.async="false";
	xmlDoc.loadXML(xmlStr);

	if(xmlDoc.parseError.errorCode != 0) {
		throw new Error('Error in XML: ' + xmlDoc.parseError.errorCode);
	}
	return xmlDoc;
}

function encryptPassword(plain){
	/*
	var crypto = document.getElementById("crypto");
	return crypto.encryptString(plain);
	*/
	//return document.crypto.encryptString(plain);
	return window.external.Task_EncryptPassword(plain);
}

//Data access functions ////////////////////////////
function addCredentials(cred){
    var db = window.external.GetXmlDbQueryCtrl();
    db.SetSPName("ENTSP_AddCredentials");
    db.CreateParamList();
    db.AddParam("username", cred.username);
    db.AddParam("password", encryptPassword(cred.password));
    db.AddParam("domainName", cred.getDomain());
    db.AddParamList();
    var resultDoc = parseXmlString(db.XMLQueryDB());

    //Return the OSDomainID for the implicitly created domain
    var params = resultDoc.getElementsByTagName("param");
    for(var i = 0; i < params.length; ++i){
    	if(params[i].getAttribute("name") == "@RETURN_VALUE"){
    		return params[i].getAttribute("value");
    	}
    }
}

function removeCredentials(id){
    var db = window.external.GetXmlDbQueryCtrl();
    db.SetSPName("ENTSP_RemoveCredentials");
    db.CreateParamList();
    db.AddParam("credentialsId", id);
    db.AddParamList();
    db.XMLQueryDB();
}

function setDomainEnabled(domainId, isEnabled){
    var db = window.external.GetXmlDbQueryCtrl();
    db.SetSPName("ENTSP_SetDomainEnabled");
    db.CreateParamList();
    db.AddParam("domainId", domainId);
    db.AddParam("enabled", isEnabled ? 1 : 0);
    db.AddParamList();
    db.XMLQueryDB();
}

function retrieveCredentials(){
    var db = window.external.GetXmlDbQueryCtrl();
    db.SetSPName("ENTSP_RetrieveCredentials");
    var resultDoc = parseXmlString(db.XMLQueryDB());

    var records = resultDoc.getElementsByTagName("record");
    var credentials = new Array();

    for(var i = 0; i < records.length; ++i){
    	var username = records[i].getElementsByTagName("Username")[0].firstChild.nodeValue;
	//we aren't decrypting the password here at the moment, because we have no need of it.
    	var password = records[i].getElementsByTagName("Password")[0].firstChild.nodeValue;
    	var id = records[i].getElementsByTagName("OSCredentialsID")[0].firstChild.nodeValue;

    	credentials.push(new Credentials(username, password, id));
    }
    return credentials;
}

function retrieveDomains(){
    var db = window.external.GetXmlDbQueryCtrl();
    db.SetSPName("ENTSP_RetrieveDomains");
    var resultDoc = parseXmlString(db.XMLQueryDB());
    var records = resultDoc.getElementsByTagName("record");

    for(var i = 0; i < records.length; ++i){
    	var name = records[i].getElementsByTagName("DistinguishedName")[0].firstChild.nodeValue;
    	var id = records[i].getElementsByTagName("OSDomainID")[0].firstChild.nodeValue;
    	var enabled = records[i].getElementsByTagName("Enabled")[0].firstChild.nodeValue;

    	domainsTable.addRow(name, enabled, id);
    }
}


//Credentials data object ///////////////////////////
function Credentials(username, password, id){
	this.username = username;
	this.password = password;
	this.id = (id == undefined) ? 0 : id;
}


Credentials.prototype.getDomain = function(){
	var pos = this.username.search(/@/);
	var domain  = "";
	if(pos < 0){
		pos = this.username.search(/\\/);
		if(pos >= 0){
			domain = this.username.substr(0, pos);
		}else{
			throw new Error(nrc_sz_ErrorMessage);
		}
	}else{
		domain = this.username.substr(pos+1);
	}
	return domain;
}

